
import re
from pathlib import Path
from dataclasses import dataclass

@dataclass
class FileEntry():
    path: str
    regex: str
    recursive: bool = True


class MultiFileReader():
    """Initializes the _MultiFileReader.
    Attributes:
        file_paths (list[FileEntry]): List of FileEntry objects containing paths and regex patterns.
        encoding (str, optional): Encoding to use for reading files. Defaults to 'utf-8'.
    """


    def __init__(self, file_paths: list[FileEntry], encoding: str = 'utf-8'):
        self._file_paths = file_paths
        self._encoding = encoding

    def get_files(self) -> list[Path]:
        output = []
        if (self._file_paths):
            for file_entry in self._file_paths:
                root = Path(file_entry.path)
                regex = re.compile(file_entry.regex)
                if (file_entry.recursive):
                    files = root.rglob("*")
                else:
                    files = root.iterdir()


                all_files = [
                    file for file in files
                    if file.is_file() and regex.search(file.name)
                ]

                # Sort by parent folder full path and name, then by file name
                sorted_files = sorted(
                    all_files,
                    key=lambda f: (str(f.parent.resolve()), f.parent.name.lower(), f.name.lower())
                )
                output.extend(sorted_files)
        return output

    def get_files_with_content(self) -> list[tuple[Path, str]]:
        output = []
        files = self.get_files()
        for file in files:
            with open(file, 'r', encoding=self._encoding) as f:
                output.append((file, f.read()))
        
        return output




from abc import abstractmethod
import json
import json5
import yaml
from deepmerge import Merger, always_merger
from dacite import from_dict
from dataclasses import asdict
from pathlib import Path


class FileMerger:

    SUPPORTED_EXTENSIONS = ['.json', '.yaml', '.yml', '.json5', '.jsonc']

    """ Merges files and returns dictionary object. Supported files are defined by SUPPORTED_EXTENSIONS
    Attributes:
        merger (Merger): Merger strategy, as default always_merger is used
    """
    def __init__(self, merger: Merger = always_merger):
        self._merger = merger
        # default strategy of always_merger
        # self._merger = Merger(
        #     [
        #         (dict, "merge"), 
        #         (list, "append"), 
        #         (set, "union")
        #     ],
        #     ["override"],
        #     ["override"]
        # )        




    @abstractmethod
    def _to_dict(self, file: Path, content) -> dict:
        if (not file.is_file()):
            return None
        
        ext = file.suffix
        if (ext not in self.SUPPORTED_EXTENSIONS):
            raise ValueError(f"Unsupported file extension: {ext}")

        if ext == '.json':
            return json.loads(content)
        elif ext in ['.json5', '.jsonc']:
            return json5.loads(content)
        elif ext in ['.yaml', '.yml']:
            return yaml.safe_load(content)

        return None

    def merge(self, files: list[tuple[Path,str]]) -> dict:
        data = None
        for file, content in files:
            file_data = None
            if content:
                file_data = self._to_dict(file, content)

            if file_data:
                if data is None:
                    data = file_data
                else:
                    self._merger.merge(data, file_data)
        return data



import re
from typing import Dict



class TokensReplacer:
    """Replaces tokens in a text with corresponding values from a dictionary.
    Attributes:
        re_start (str): The starting delimiter for tokens (regex). Default is "{".
        re_end (str): The ending delimiter for tokens (regex). Default is "}".
        re_escape (str): The escape character for tokens (regex). Default is "\\".
    """
    def __init__(self, re_start: str = r"\{", re_end: str = r"\}", re_escape: str = "\\\\"):


        # Match unescaped tokens across multiple lines
        self._pattern = re.compile(
            rf'(?<!{re_escape})({re_start})(?P<token>.*?)({re_end})',
            flags=re.DOTALL
        )

        # Match escaped tokens to clean them up later
        self._escaped_pattern = re.compile(
            rf'{re_escape}({re_start}.*?{re_end})',
            flags=re.DOTALL
        )

    def replace(self, text: str, tokens: Dict[str, str]) -> str:
        def replacer(match):
            token = match.group("token")
            if token not in tokens:
                raise KeyError(f"Missing replacement for token: '{token}'")
            return tokens[token]

        # Replace unescaped tokens
        result = self._pattern.sub(replacer, text)

        # Remove escape character from escaped tokens: \{token} → {token}
        result = self._escaped_pattern.sub(lambda m: m.group(1), result)

        return result


import json
from pathlib import Path
from typing import Dict, Any

class ConfigReader:
    def __init__(self, file_path: str):
        file_path = Path(file_path)

        self._data = {}
        json_data = None

        with open(file_path, "r", encoding="utf-8") as f:
            json_data = json.load(f)

        if (json_data):
            for item in json_data["tests"]:
                self._data[item["name"]] = item["expected"]
        
    def get_expectations(self, test_name) -> Any:
        return self._data.get(test_name, None)



import re
from dataclasses import dataclass
from pathlib import Path
from typing import List




@dataclass
class CommandEntry:
    file: str
    version: str
    command: str


class FileCommandSplitter:
    """Parses text files that contain VERSION and COMMAND sections. It splits files to parts version-command.
    """

    def __init__(self,
                 re_version = r"^\s*--\s*VERSION:\s*(?P<version>.+)\s*$",
                 re_command = r"^\s*--\s*COMMAND\s*$"
                 ):
        self._version_re = re.compile(re_version)
        self._command_re = re.compile(re_command)

    @classmethod
    def version_key(cls, v: str):
        if not v:
            return (0,)
        parts = re.split(r"[^0-9A-Za-z]+", v)
        key = []
        for p in parts:
            if p.isdigit():
                key.append(int(p))
            else:
                key.append(p.lower())
        return tuple(key)


    def _parse_text(self, file_path: str, text: str) -> List[CommandEntry]:
        lines = text.splitlines()
        current_version = ""
        in_command = False
        command_lines: List[str] = []
        results: List[CommandEntry] = []

        for line in lines:
            mver = self._version_re.match(line)
            if mver:
                # If we're inside a command, close it before switching versions
                if in_command:
                    command_text = "\n".join(command_lines).strip()
                    if command_text:
                        results.append(CommandEntry(file=file_path, version=current_version, command=command_text))
                    command_lines = []
                    in_command = False
                # set current version
                current_version = mver.group('version').strip()
                continue

            if self._command_re.match(line):
                # When we see a COMMAND marker, close any existing command and
                # start a new one. This makes each COMMAND marker the delimiter
                # that ends the previous command and begins the next.
                if in_command:
                    command_text = "\n".join(command_lines).strip()
                    if command_text:
                        results.append(CommandEntry(file=file_path, version=current_version, command=command_text))
                    command_lines = []
                    in_command = True
                else:
                    in_command = True
                continue

            if in_command:
                command_lines.append(line)

        # If file ends while in a command, close it
        if in_command:
            command_text = "\n".join(command_lines).strip()
            if command_text:
                results.append(CommandEntry(file=file_path, version=current_version, command=command_text))

        return results

    def _read_file(self, path: Path) -> List[CommandEntry]:
        """Read a single file and parse its content."""
        with path.open("r", encoding="utf-8") as f:
            text = f.read()
        entries = self._parse_text(path.absolute().as_posix(), text)

        return entries

    def read_files(self, paths: List[Path]) -> List[CommandEntry]:
        """Read multiple files and return combined list of (version, command).

        Results are returned in the order files are provided; callers can sort
        them as needed.
        """
        out: List[CommandEntry] = []
        for p in paths:
            out.extend(self._read_file(p))

        # Sort by semantic version ascending, then by file path ascending
        out.sort(key=lambda e: (self.version_key(e.version), e.file or ''))
        return out
